@@ -1,7 +1,7 @@
 /*
- * @(#) $Id: jpeg.c 287 2017-01-21 19:15:36Z leres $ (XSE)
+ * @(#) $Id: jpeg.c 334 2023-09-25 20:59:36Z leres $ (XSE)
  *
- * Copyright (c) 1999, 2000, 2008, 2009, 2010, 2015, 2017
+ * Copyright (c) 1999, 2000, 2008, 2009, 2010, 2015, 2017, 2023
  *	Craig Leres
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,6 +25,9 @@
 #include "os-proto.h"
 #endif
 
+#ifdef HAVE_LIBEXIF
+#include "exif.h"
+#endif
 #include "jpeg.h"
 
 /* Define a few JPEG markers */
@@ -40,7 +43,8 @@ static int get(FILE *, u_char *);
 static int mustbe(FILE *, u_char);
 
 int
-parsejpeg(const char *image, int *wp, int *hp, struct timespec *mtimp)
+parsejpeg(const char *image, int *wp, int *hp, struct timespec *mtimp,
+    char *location, size_t locationsize)
 {
 	int status;
 	FILE *f;
@@ -63,6 +67,13 @@ parsejpeg(const char *image, int *wp, int *hp, struct timespec *mtimp)
 		status = jpeg(f, wp, hp);
 	else
 		status = 0;
+
+	if (location != NULL && locationsize > 0) {
+		location[0] = '\0';
+#ifdef HAVE_LIBEXIF
+		(void)parseexif(f, location, locationsize);
+#endif
+	}
 	fclose(f);
 	return (status);
 }
