@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: webindex.c 322 2023-01-14 21:42:39Z leres $ (XSE)
+ * @(#) $Id: webindex.c 340 2023-09-28 03:02:07Z leres $ (XSE)
  *
  * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016, 2017, 2018, 2019, 2022, 2023
  *	Craig Leres
@@ -93,6 +93,7 @@ struct webfile {
 	struct webfile *w_tn;		/* configured thumbnail */
 	mode_t w_mode;			/* file mode from stat() */
 	char w_name[1024];		/* simple filename */
+	char w_location [32];		/* GPS location */
 };
 
 /* Flags */
@@ -115,10 +116,6 @@ char *reason;
 const char defindexfn[] = "index.html";
 const char *configfn = "webindex.conf";
 const char defdatefmt[] = "%l:%M%p";
-const char deftable[] = "<table>\n";
-const char debugtable[] = "<table style=\"border: 1px;\">\n";
-const char deftr[] =
-    "<tr style=\"text-align: center; vertical-align: top;\">\n";
 
 extern char *optarg;
 extern int optind, opterr, optopt;
@@ -129,6 +126,35 @@ static const char tnlowsuffix[] = "_t.jpg";
 static const char tnuppersuffix[] = "_t.JPG";
 static int tnlowsuffixlen = sizeof(tnlowsuffix) - 1;
 static int tnuppersuffixlen = sizeof(tnuppersuffix) - 1;
+static const char gmapsfmt[] = "https://www.google.com/maps/place/%s";
+static const char stylefmt[] =
+	"<style>\n"
+	"table.w tr, table.w td {\n"
+	"    text-align: center;\n"
+	"    vertical-align: top;\n"
+	"%s}\n"
+	/* https://stackoverflow.com/a/65083940 */
+	".location {\n"
+	"    box-sizing: border-box;\n"
+	"    position: relative;\n"
+	"    display: inline-block;\n"
+	"    width: 16px;\n"
+	"    height: 16px;\n"
+	"    border: 5px solid blue;\n"
+	"    border-radius: 100%%;\n"
+	"}\n"
+	".location::after {\n"
+	"    position: absolute;\n"
+	"    top: 100%%;\n"
+	"    left: 50%%;\n"
+	"    margin: 1px 0 0 -6px;\n"
+	"    display: block;\n"
+	"    content: '';\n"
+	"    border: 6px solid transparent;\n"
+	"    border-top: 10px solid blue;\n"
+	"    border-bottom: none;\n"
+	"}\n"
+	"</style>\n";
 
 static struct s2v suffix2type[] = {
 	{ "avi",	T_AVI },
@@ -300,10 +326,8 @@ process(const char *dname)
 	ind_tim.tv_nsec = 0;
 	force = 0;
 	indexfn = defindexfn;
-	table = deftable;
-	if (debug)
-		table = debugtable;
-	tr = deftr;
+	table = "<table class=\"w\">\n";
+	tr = "<tr>";
 	tail = "\n";
 	otz = NULL;
 	savedtz = 0;
@@ -731,7 +755,7 @@ void
 processfile(FILE *f, struct webfile *wp, struct cf *cf,
     const char *table, const char *tr, const char *indexfn)
 {
-	char *cp, *name;
+	char *cp, *cp2, *name;
 	const char *p, *icon, *alt, *linkpostfix;
 	struct webfile *wp2;
 	struct list *lp;
@@ -740,6 +764,8 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 	struct tm *tm;
 	char buf[132];
 	char target[256];
+	char location[32];
+	char url[64];
 
 	if (WEBFILE_ISDIR(wp) && (cf->c_flags & C_APPDIRINDEXFN) != 0)
 		linkpostfix = indexfn;
@@ -922,6 +948,25 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 		if (*cp != '\0')
 			fprintf(f, "%s<td><small>%s</small></td></tr>\n",
 			    tr, cp);
+
+		/* location */
+		if (wp->w_location[0] != '\0') {
+			/* url location */
+			strlcpy(location, wp->w_location, sizeof(location));
+			cp2 = strchr(location, ' ');
+			if (cp2 != NULL) {
+				i = wp->w_location - cp2;
+				j = strlen(cp + 1) + 1;
+				memmove(cp2 + 3, cp2 + 1, j);
+				cp2[0] = '%';
+				cp2[1] = '2';
+				cp2[2] = '0';
+			}
+			(void)snprintf(url, sizeof(url), gmapsfmt, location);
+			fprintf(f, "%s<td><a href=\"%s\" "
+			    "class=\"location\"></a></td></tr>\n", tr, url);
+		}
+
 	}
 
 	/* Look for a caption */
@@ -1018,6 +1063,8 @@ void
 parsethumbnail(struct cf *cf, struct webfile *wp)
 {
 	int v, flags;
+	char *location;
+	size_t locationsize;
 
 	flags = 0;
 	switch (wp->w_type) {
@@ -1028,18 +1075,26 @@ parsethumbnail(struct cf *cf, struct webfile *wp)
 		break;
 
 	case T_GIF:
-		(void)parsegif(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
+		(void)parsegif(wp->w_name, &wp->w_w, &wp->w_h, NULL);
 		flags |= W_HAVETN;
 		break;
 
 	case T_JPEG:
-		(void)parsejpeg(wp->w_name, &wp->w_w, &wp->w_h, NULL);
+		if ((cf->c_flags & C_NO_LOCATION) == 0) {
+			location = wp->w_location;
+			locationsize = sizeof(wp->w_location);
+		} else {
+			location = NULL;
+			locationsize = 0;
+		}
+		(void)parsejpeg(wp->w_name, &wp->w_w, &wp->w_h, NULL,
+		    location, locationsize);
 		flags |= W_HAVETN;
 		break;
 
 	case T_PBM:
 #ifdef HAVE_LIBPPM
-		(void)parsepbm(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
+		(void)parsepbm(wp->w_name, &wp->w_w, &wp->w_h, NULL);
 #endif
 		flags |= W_HAVETN;
 		break;
@@ -1051,18 +1106,18 @@ parsethumbnail(struct cf *cf, struct webfile *wp)
 
 	case T_TIFF:
 #ifdef HAVE_LIBTIFF
-		(void)parsetiff(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
+		(void)parsetiff(wp->w_name, &wp->w_w, &wp->w_h, NULL);
 #endif
 		flags |= W_HAVETN;
 		break;
 
 	case T_XBM:
-		(void)parsexbm(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
+		(void)parsexbm(wp->w_name, &wp->w_w, &wp->w_h, NULL);
 		flags |= W_HAVETN;
 		break;
 
 	case T_XWD:
-		(void)parsexwd(wp->w_name,  &wp->w_w, &wp->w_h, NULL);
+		(void)parsexwd(wp->w_name, &wp->w_w, &wp->w_h, NULL);
 		flags |= W_HAVETN;
 		break;
 
@@ -1146,7 +1201,7 @@ reapthumbnails(const char *dir, struct cf *cf,
 
 			/* Get thumbnail dimensions and timestamp */
 			sprintf(temp, "%s/%s", dir, dp->d_name);
-			if (!parsejpeg(temp, &w, &h, &wp->w_tn_tim)) {
+			if (!parsejpeg(temp, &w, &h, &wp->w_tn_tim, NULL, 0)) {
 				++force;
 				reason = " (parsejpeg)";
 			} else if (wp->w_tn_tim.tv_sec == 0) {
@@ -1230,16 +1285,19 @@ header(FILE *f, struct cf *cf)
 		zone[0] = '+';
 	(void)sprintf(zone + 1, "%04d", z);
 #endif
+
+
 	fprintf(f, "<!DOCTYPE html>\n");
 	fprintf(f, "<!-- %s\n    Version %s (this file created on %s %s) -->\n",
 	    magic, version, date, zone);
 	fprintf(f, "<html lang=\"en\"><head>\n");
+	fprintf(f, stylefmt, debug ? "    border: 1px solid;\n" : "");
 	if (cf->c_favicon != NULL)
 		fprintf(f, "<link rel=\"shortcut icon\" href=\"%s\">\n",
 		    cf->c_favicon);
 	if (cf->c_css != NULL)
 		fprintf(f, "<link rel=\"stylesheet\" type=\"text/css\""
-		    " href=\"%s\" />\n", cf->c_css);
+		    " href=\"%s\">\n", cf->c_css);
 	fprintf(f, "<meta http-equiv=\"Content-type\""
 	    " content=\"text/html;charset=UTF-8\">\n");
 	fprintf(f, "<meta name=\"viewport\""
@@ -1341,7 +1399,7 @@ createthumb(struct webfile *wp, const char *fn, struct cf *cf)
 		fprintf(stderr, "\n");
 		return (0);
 	}
-	if (parsejpeg(fn, &w, &h, NULL)) {
+	if (parsejpeg(fn, &w, &h, NULL, NULL, 0)) {
 		wp->w_tnw = w;
 		wp->w_tnh = h;
 	} else {
