@@ -1,4 +1,4 @@
-/* @(#) $Id: webindex.h 287 2017-01-21 19:15:36Z leres $ (XSE) */
+/* @(#) $Id: webindex.h 322 2023-01-14 21:42:39Z leres $ (XSE) */
 #ifndef webindex_h
 #define webindex_h
 
@@ -45,7 +45,7 @@ extern int debug;
 extern int recurse;
 extern int verbose;
 extern int errors;
-extern const char configfn[];
+extern const char *configfn;
 extern char prefix[];
 extern const char version[];
 
